@@ -137,15 +137,15 @@ def distances(arguments):
 
 
     from kmerdb import fileutil, distance, config
-    n = len(arguments.kdb)
+    n = len(arguments.input)
 
 
-    if len(arguments.kdb) > 1:
-        files = list(map(lambda f: fileutil.open(f, 'r'), arguments.kdb))
+    if len(arguments.input) > 1:
+        files = list(map(lambda f: fileutil.open(f, 'r'), arguments.input))
         logger.debug("Files: {0}".format(files))
         if arguments.k is None:
             arguments.k = files[0].k
-        if not all(os.path.splitext(kdb)[-1] == ".kdb" for kdb in arguments.kdb):
+        if not all(os.path.splitext(kdb)[-1] == ".kdb" for kdb in arguments.input):
             raise IOError("One or more parseable .kdb filepaths did not end in '.kdb'")
         if not all(kdbrdr.k == arguments.k for kdbrdr in files):
             logger.error("Files: {0}".format(files))
@@ -174,8 +174,8 @@ def distances(arguments):
             raise RuntimeError("Raw profile shape (a Numpy array) doesn't match expected dimensions")
 
         df = pd.DataFrame(profiles, columns=column_names)
-    elif len(arguments.kdb) == 1 and (arguments.kdb[0] == "/dev/stdin" or arguments.kdb[0] == "STDIN"):
-        logger.info("Hidden: 1 argument. Reading input from stdin")
+    elif len(arguments.input) == 0 or (len(arguments.input) == 1 and (arguments.input[0] == "STDIN" or arguments.input[0] == "/dev/stdin")):
+        logger.info("Reading input as tsv/csv from STDIN")
         try:
             df = pd.read_csv(sys.stdin, sep=arguments.delimiter)
         except pd.errors.EmptyDataError as e:
@@ -185,10 +185,10 @@ def distances(arguments):
         profiles = np.array(df)
         column_names = list(df.columns)
         n = len(column_names)
-    elif len(arguments.kdb) == 1 and os.path.splitext(arguments.kdb[0])[-1] == ".tsv":
+    elif len(arguments.input) == 1 and (os.path.splitext(arguments.input[0])[-1] == ".tsv" or os.path.splitext(arguments.input[0])[-1] == ".csv"):
         logger.info("Hidden: 1 argument. Reading input as tsv")
         try:
-            df = pd.read_csv(arguments.kdb[0], sep=arguments.delimiter)
+            df = pd.read_csv(arguments.input[0], sep=arguments.delimiter)
         except pd.errors.EmptyDataError as e:
             logger.error(e)
             logger.error("Pandas error on DataFrame reading. Perhaps a null dataset being read?")
@@ -196,16 +196,16 @@ def distances(arguments):
         profiles = np.array(df)
         column_names = list(df.columns)
         n = len(column_names)
-    elif len(arguments.kdb) == 1 and os.path.splitext(arguments.kdb)[-1] == ".kdb":
+    elif len(arguments.input) == 1 and os.path.splitext(arguments.input)[-1] == ".kdb":
         logger.error("kdb distance requires more than one .kdb file as positional inputs")
         sys.exit(1)
     else:
-        logger.error("bin/kdb.distances() received {0} arguments as input, which were not supported.".format(len(arguments.kdb)))
+        logger.error("bin/kdb.distances() received {0} arguments as input, which were not supported.".format(len(arguments.input)))
         sys.exit(1)
 
     # Masthead stuff
     sys.stderr.write(config.DEFAULT_MASTHEAD)
-    if logger.level == logging.DEBUG:
+    if logger.level == logging.DEBUG or logger.level == logging.INFO:
         sys.stderr.write(config.DEBUG_MASTHEAD)
     sys.stderr.write(config.DISTANCE_MASTHEAD)
 
@@ -222,10 +222,10 @@ def distances(arguments):
                     data[i][j] = None
                 elif i < j:
                     if arguments.metric == "correlation":
-                        data[i][j] = distance.correlation(arguments.kdb[i], arguments.kdb[j])
+                        data[i][j] = distance.correlation(arguments.input[i], arguments.input[j])
                         # data[i][j] = distance.correlation(arguments.kdb[i], arguments.kdb[j])
                     elif arguments.metric == "euclidean":
-                        data[i][j] = distance.euclidean(arguments.kdb[i], arguments.kdb[j])
+                        data[i][j] = distance.euclidean(arguments.input[i], arguments.input[j])
                     elif arguments.metric == "spearman":
                         cor, pval = distance.spearman(profiles[i], profiles[j])
                         # FIXME! also print pval matrices
@@ -248,9 +248,9 @@ def distances(arguments):
     else:
         dist = pdist(np.transpose(profiles), metric=arguments.metric)
         dist = squareform(dist)
-        if arguments.metric == "correlation":
-            ones = np.ones(dist.shape, dtype="int64")
-            dist = np.subtract(ones, dist)
+        # if arguments.metric == "correlation":
+        #     ones = np.ones(dist.shape, dtype="int64")
+        #     dist = np.subtract(ones, dist)
     if dist.shape == (2,2):
         print(dist[0][1])
     else:
@@ -279,11 +279,11 @@ def get_matrix(arguments):
     from kmerdb import fileutil, config
 
     
-    if len(arguments.kdb) > 1:
-        files = list(map(lambda f: fileutil.open(f, 'r'), arguments.kdb))
+    if len(arguments.input) > 1:
+        files = list(map(lambda f: fileutil.open(f, 'r'), arguments.input))
         if arguments.k is None:
             arguments.k = files[0].k
-        if not all(os.path.splitext(kdb)[-1] == ".kdb" for kdb in arguments.kdb):
+        if not all(os.path.splitext(kdb)[-1] == ".kdb" for kdb in arguments.input):
             raise IOError("One or more parseable .kdb filepaths did not end in '.kdb'")
         if not all(kdbrdr.k == arguments.k for kdbrdr in files):
             logger.error("Files: {0}".format(files))
@@ -302,8 +302,8 @@ def get_matrix(arguments):
 
         column_names = list(map(lambda kdbrdr: os.path.basename(kdbrdr._filepath).split(".")[0], files))
         df = pd.DataFrame(profiles, columns=column_names)
-    elif len(arguments.kdb) == 1 and (arguments.kdb[0] == "STDIN" or arguments.kdb[0] == "/dev/stdin"):
-        logger.info("Hidden: 1 argument. Reading input as tsv from STDIN")
+    elif len(arguments.input) == 0 or (len(arguments.input) == 1 and (arguments.input[0] == "STDIN" or arguments.input[0] == "/dev/stdin")):
+        logger.info("Reading input as tsv/csv from STDIN")
         try:
             df = pd.read_csv(sys.stdin, sep=arguments.delimiter)
         except pd.errors.EmptyDataError as e:
@@ -311,21 +311,22 @@ def get_matrix(arguments):
             logger.error("Pandas error on DataFrame reading. Perhaps a null dataset being read?")
             sys.exit(1)
         column_names = list(df.columns)
-    elif len(arguments.kdb) == 1 and (os.path.splitext(arguments.kdb[0])[-1] == ".tsv"):
-        logger.info("Hidden: 1 argument. Reading input as tsv from {0}".format(arguments.kdb[0]))
+    elif len(arguments.input) == 1 and (os.path.splitext(arguments.input[0])[-1] == ".tsv" or os.path.splitext(arguments.input[0])[-1] == ".csv"):
+        logger.info("Reading input file as tsv/csv")
         try:
-            df = pd.read_csv(arguments.kdb[0], sep=arguments.delimiter)
+            df = pd.read_csv(arguments.input[0], sep=arguments.delimiter)
         except pd.errors.EmptyDataError as e:
             logger.error(e)
             logger.error("Pandas error on DataFrame reading. Perhaps a null dataset being read?")
             sys.exit(1)
         column_names = list(df.columns)
     else:
-        logger.error("bin/kdb.get_matrix() received {0} arguments as input, and this is not supported.".format(len(arguments.kdb)))
-        logger.error(arguments.kdb)
+        logger.error(arguments)
+        logger.error("bin/kdb.get_matrix() received {0} arguments as input, and this is not supported.".format(len(arguments.input)))
+        logger.error(arguments.input)
         sys.exit(1)
     sys.stderr.write(config.DEFAULT_MASTHEAD)
-    if logger.level == logging.DEBUG:
+    if logger.level == logging.DEBUG or logger.level == logging.INFO:
         sys.stderr.write(config.DEBUG_MASTHEAD)
     sys.stderr.write(config.MATRIX_MASTHEAD)
 
@@ -376,6 +377,7 @@ def get_matrix(arguments):
         except PackageNotInstalledError as e:
             logger.error(e)
             logger.error("One or more R packages were not installed. Please see the install documentation about installing the associated R packages")
+            logger.error("Use -vv for suggested installation instructions.")
             sys.exit(1)
         #normalized.to_csv(sys.stdout, sep=arguments.delimiter, index=arguments.with_index)
 
@@ -488,7 +490,7 @@ def kmeans(arguments):
         sys.exit(1)
 
     sys.stderr.write(config.DEFAULT_MASTHEAD) # Not working
-    if logger.level == logging.DEBUG: # Not working
+    if logger.level == logging.DEBUG or logger.level == logging.INFO: # Not working
         sys.stderr.write(config.DEBUG_MASTHEAD)
     sys.stderr.write(config.KMEANS_MASTHEAD) # Not working
 
@@ -567,9 +569,9 @@ def kmeans(arguments):
 
         results += [metrics.silhouette_score(df, final_kmeans.labels_, metric="euclidean", sample_size=300,)]
 
-        formatter_result = ("{:9s}\t{:.3f}s\t{:.0f}\t{:.3f}\t{:.3f}"
+        formatter_result = ("\n{:9s}\t{:.3f}s\t{:.0f}\t{:.3f}\t{:.3f}"
                             "\t{:.3f}\t{:.3f}\t{:.3f}\t{:.3f}\n\n\n")
-        sys.stderr.write("\t".join(["Name", "Total_time", "Inertia", "Homogeneity_score", "Completeness_score", "V_measure_score", "Adj_rand_score", "Adj_mutual_info_score", "Silhouette"]) + "\n")
+        sys.stderr.write("\n" + "\t".join(["Name", "Total_time", "Inertia", "Homogeneity_score", "Completeness_score", "V_measure_score", "Adj_rand_score", "Adj_mutual_info_score", "Silhouette"]) + "\n")
         sys.stderr.write(formatter_result.format(*results))
 
         #kmer_pca = pca(df, scale=False) # It's already been normalized (scaled) in get_matrix()
@@ -651,7 +653,7 @@ def hierarchical(arguments):
         sys.exit(1)
 
     sys.stderr.write(config.DEFAULT_MASTHEAD)
-    if logger.level == logging.DEBUG:
+    if logger.level == logging.DEBUG or logger.level == logging.INFO:
         sys.stderr.write(config.DEBUG_MASTHEAD)
     sys.stderr.write(config.HIERARCHICAL_MASTHEAD)
     
@@ -870,18 +872,21 @@ def profile(arguments):
     from multiprocessing import Pool
     import json
     import time
-    from kmerdb import parse, fileutil, kmer, database
+    import numpy as np
+    from kmerdb import parse, fileutil, kmer, util
     from kmerdb.config import VERSION
 
     logger.debug(arguments)
 
-
-    
+    if arguments.parallel <= 0 or arguments.parallel > cpu_count()+1:
+        raise argparse.ArgumentError("-p, --parallel must be a valid processor count.")
     if os.path.splitext(arguments.kdb)[-1] != ".kdb":
         raise IOError("Destination .kdb filepath does not end in '.kdb'")
     
     file_metadata = []
-    tempdbs = []
+    total_kmers = 4**arguments.k
+    final_counts = np.zeros(total_kmers, dtype='uint')
+    all_kmer_metadata = list([] for x in range(total_kmers))
     logger.info("Parsing {0} sequence files to generate a composite k-mer profile...".format(len(list(arguments.seqfile))))
     nullomers = set()
     pool = Pool(processes=arguments.parallel)
@@ -891,175 +896,72 @@ def profile(arguments):
         #logger.info("Processing fastas in parallel")
         logger.debug("Parallel (if specified) mapping the kmerdb.parse.parsefile() method to the seqfile iterable")
         logger.debug("In other words, running the kmerdb.parse.parsefile() method many times on each file multiply specified via the CLI")
-        list_of_dbs = pool.map(infile.parsefile, arguments.seqfile)
+        data = pool.map(infile.parsefile, arguments.seqfile)
+
     else:
         logger.info("Processing (some) fastqs with alternate parallelization schema. WARNING: UNTESTED")
         logger.warning("WARNING: UNTESTED")
         logger.debug("Mapping the kmerdb.parse.parsefile() method to the seqfile iterable")
-        list_of_dbs = list(map(infile.parsefile, arguments.seqfile))
-    logger.info("Generating list of temporary database handles")
+        data = list(map(infile.parsefile, arguments.seqfile))
     # list_of_dbs is a list of tuples returned from the mapping (either via pool.map or via regular Pythonic map) of the .parsefile() method
     # each tuple is a triple of the database handle, the metadata, and the set/list of nullomer ids
-    for db, m, n in list_of_dbs:
-        logger.debug("Appending metadata...")
-        file_metadata.append(m)
-        db = database.PostgresKdb(arguments.k, arguments.postgres_connection, tablename=db, filename=m["filename"])
-        tempdbs.append(db)
 
-        # Calculating nullomer additions
-        lenBefore = len(nullomers)
-        nullomers = nullomers.union(n)
-        lenAfter = len(nullomers)
-        newNullomers = lenAfter - lenBefore
-        logger.info("{0} contributed {1} unique nullomers to the profile".format(m["filename"], len(n)))
-        logger.info("Completed transfer of k-mers from {0} into PostgreSQL database.".format(m["filename"]))
+
+    # Complete collating of counts across files
+    for d in data:
+        final_counts = final_counts + d[0] # Add the counts to the zeroes array
+        if arguments.all_metadata:
+            
+            all_kmer_metadata = util.merge_metadata_lists(arguments.k, all_kmer_metadata, d[3])
+    unique_kmers = int(np.count_nonzero(final_counts))
+    total_nullomers = total_kmers - unique_kmers
+    all_observed_kmers = int(np.sum(final_counts))
+    
+
     logger.info("Initial counting process complete, creating BGZF format file (.kdb)...")
     logger.info("Formatting master metadata dictionary...")
     metadata=OrderedDict({
         "version": VERSION,
         "metadata_blocks": 1,
         "k": arguments.k,
-        "total_kmers": sum(list(map(lambda x: x["total_kmers"], file_metadata))),
-        "unique_kmers": 4**arguments.k - len(nullomers),
+        "total_kmers": all_observed_kmers,
+        "unique_kmers": unique_kmers,
         "metadata": False,
         "tags": [],
-        "files": file_metadata
+        "files": [d[1] for d in data]
     })
         
 
 
     
-    metadata_bytes = bgzf._as_bytes(yaml.dump(metadata))
-    # The number of metadata blocks is the number of bytes of the header block(s) / the number of bytes per block in the BGZF specification
-    metadata["metadata_blocks"] = math.ceil( sys.getsizeof(metadata_bytes) / ( 2**16 ) ) # First estimate
-    metadata_bytes = bgzf._as_bytes(yaml.dump(metadata))
-    metadata["metadata_blocks"] = math.ceil( sys.getsizeof(metadata_bytes) / ( 2**16 ) ) # Second estimate
 
     logger.info("Collapsing the k-mer counts across the various input files into the final kdb file '{0}'".format(arguments.kdb)) 
     kdb_out = fileutil.open(arguments.kdb, 'wb', metadata=metadata)
     try:
-        x = 0
-        iterating = True
-        while iterating:
-            try:
-                logger.debug("Collating counts across all files for the {0} k-mer".format(x))
-                kmer_dbrecs_per_file = list(map(next, tempdbs)) # Need to rename this variable
+        x=0
+        for i, count in enumerate(final_counts):
+            seq = kmer.id_to_kmer(i, arguments.k)
+            kmer_metadata = kmer.neighbors(seq, arguments.k) # metadata is initialized by the neighbors
+            if arguments.all_metadata:
+                reads = []
+                starts = []
+                reverses = []
+                for read, start, reverse in all_kmer_metadata[i]:
+                    reads.append(read)
+                    starts.append(start)
+                    reverses.append(reverse)
+                kmer_metadata["reads"] = reads
+                kmer_metadata["starts"] = starts
+                kmer_metadata["reverses"] = reverses
+            kdb_out.write("{0}\t{1}\t{2}\n".format(i, count, kmer_metadata))
+            x = i
+        logger.info("Wrote 4^k = {0} k-mer counts + neighbors to the .kdb file.".format(x))
 
-                # Unstable code
-                #print(kmer_dbrecs_per_file)
-
-                # raise RuntimeError("HOW DOES THIS HAVE NONE")
-                if len(kmer_dbrecs_per_file):
-                    i = kmer_dbrecs_per_file[0][0] - 1 # Remove 1 for the Sqlite zero-based indexing
-                    count = sum([x[1] for x in kmer_dbrecs_per_file]) # The 1th element is the k-mer count, the 0th is the id
-                    if arguments.verbose == 2:
-                        sys.stderr.write("K-mer counts: {0} = {1}\n".format(list(map(lambda x: x[1], kmer_dbrecs_per_file)), count))
-                    if count == 0 and arguments.sparse is True:
-                        pass
-                    else:
-                        seq = kmer.id_to_kmer(i, arguments.k)
-                        kmer_metadata = kmer.neighbors(seq, arguments.k) # metadata is initialized by the neighbors
-                        # metadata now has three additional properties, based on the total number of times this k-mer occurred. Eventually the dimension of these new properties should match the count.
-                        if arguments.all_metadata:
-
-
-                            seqids = [x[4] for x in kmer_dbrecs_per_file]
-                            starts = [x[2] for x in kmer_dbrecs_per_file]
-                            reverses = [x[3] for x in kmer_dbrecs_per_file]
-
-
-                            if len(reverses) == 0:
-                                logger.error("REVERSES: {0}".format(reverses[0]))
-                                raise RuntimeError("reverses: IS THIS INCORRECT?")
-                            elif len(starts) == 0:
-                                logger.error("STARTS: {0}".format(starts[0]))
-                                raise RuntimeError("starts: IS THIS INCORRECT?")
-                            elif len(seqids) == 0:
-                                logger.error("SEQIDS: {0}".format(seqids[0]))
-                                raise RuntimeError("seqids: IS THIS INCORRECT?")
-                            elif len(seqids) == 1 and type(seqids) is list and type(seqids[0]) is list:
-                                seqids = seqids[0]
-                            elif len(starts) == 1 and type(starts) is list and type(starts[0]) is list:
-                                starts = starts[0]
-                            elif len(reverses) == 1 and type(reverses) is list and type(reverses[0]) is list:
-                                reverses = reverses[0]
-                            
-                            if "seqids" in kmer_metadata.keys():
-                                kmer_metadata["seqids"] += seqids
-                            else:
-                                kmer_metadata["seqids"] = seqids
-                            if "starts" in kmer_metadata.keys():
-                                kmer_metadata["starts"] += starts
-                            else:
-                                kmer_metadata["starts"] = starts
-                            if "reverses" in kmer_metadata.keys():
-                                kmer_metadata["reverses"] += reverses
-                            else:
-                                kmer_metadata["reverses"] = reverses
-                        else:
-                            kmer_metadata["seqids"]   = []
-                            kmer_metadata["starts"]   = []
-                            kmer_metadata["reverses"] = []
-                        if type(kmer_metadata["starts"]) is str:
-                            raise TypeError("kmerdb profile could not decode start sites from its preQLite3 database.")
-                        elif type(kmer_metadata["starts"]) is list and all(type(x) is int for x in kmer_metadata["starts"]):
-                            if arguments.verbose == 2:
-                                sys.stderr.write("Parsed {0} k-mer start sites from this sequence.".format(len(kmer_metadata["starts"])))
-                        elif type(kmer_metadata["starts"]) is dict:
-                            logger.debug("Don't know how starts became a dictionary, but this will not parse correctly. RuntimeError")
-                            raise RuntimeError("The implicit type of the Text blob in the Postgres database has changed, and will not parse correctly in kmerdb, rerun with verbose")
-                        elif type(kmer_metadata["seqids"]) is list and all(type(x) is str for x in kmer_metadata["seqids"]):
-                            if arguments.verbose == 2:
-                                sys.stderr.write("Parsed {0} sequence ids associated with this k-mer.".format(len(kmer_metadata["seqids"])))
-                        elif type(kmer_metadata["seqids"]) is dict:
-                            logger.debug("Don't know how seqids became a dictionary, but this will not parse correctly. RuntimeError")
-                            raise RuntimeError("The implicit type of the Text blob in the Postgres database has changed, and will not parse correctly in kmerdb, rerun with verbose")
-                        elif type(kmer_metadata["reverses"]) is str:
-                            raise TypeError("kmerdb profile could not decode strand information from its PostgreSQL database.")
-                        elif type(kmer_metadata["reverses"]) is list and all(type(x) is bool for x in kmer_metadata["reverses"]):
-                            if arguments.verbose == 2:
-                                sys.stderr.write("Parsed {0} reverse? bools associated with this k-mer.".format(len(kmer_metadata["seqids"])))
-                        elif type(kmer_metadata["reverses"]) is dict:
-                            logger.debug("Don't know how reverses became a dictionary, but this will not parse correctly. RuntimeError")
-                            raise RuntimeError("The implicit type of the Text blob in the Postgres database has changed, and will not parse correctly in kmerdb, rerun with verbose")
-                        elif not all(type(x) is bool for x in kmer_metadata["reverses"]):
-                            logger.error("kmer metadata: {0}".format(kmer_metadata))
-                            logger.error("number of k-mer elements: {0}".format(len(kmer_metadata.values())))
-                            logger.error(list(set(type(x) for x in kmer_metadata["reverses"])))
-                            raise TypeError("Not all reverse bools were boolean")
-                        elif count == 0:
-                            n += 1
-                        kdb_out.write("{0}\t{1}\t{2}\n".format(i, count, kmer_metadata))
-                    x += 1
-                else:
-                    iterating = False
-            except StopIteration as e:
-                logger.warn("Shouldn't have encountered StopIteration error...")
-                logger.warn("Continuing anyways...")
-                iterating = False
-        logger.info("Completed the transfer of data from the {0} temporary SQLite3 databases to the kdb file '{1}'".format(len(tempdbs), arguments.kdb))
         logger.info("Done")
     finally:
-        import shutil
-        from psycopg2 import sql
         kdb_out._write_block(kdb_out._buffer)
         kdb_out._handle.flush()
         kdb_out._handle.close()
-        for db in tempdbs:
-            if not arguments.keep_db:
-                #os.unlink(db.filepath)
-                with db.conn.begin():
-                    db.conn.execute("DROP TABLE {}".format(db._tablename))
-                # else:
-            #     kdbfile = arguments.kdb
-            #     kdbbasename = arguments.kdb.rstrip(".kdb")
-            #     kdbsqlite = kdbbasename + ".sqlite3"
-            #     shutil.move(db.filepath, kdbsqlite)
-            #     sys.stderr.write("    Database file retained as    '{0}'".format(kdbsqlite))
-
-            if db.conn is not None:
-                db.conn.close()
-            db._engine.dispose()
 
     
 # def gen_hist(arguments):
@@ -1108,6 +1010,7 @@ def citation_info():
         else:
             sys.stderr.write("Printing citation notice to stderr. This will not interfere with the execution of the program in any way. Please see CITATION_FAQ.md for any questions.\n")
             sys.stderr.write(citation + "\n\n\n")
+            sys.stderr.write("'kmerdb citation' to silence.\n")
     else:
         raise IOError("Cannot locate the extra package data file 'kmerdb/CITATION', which should have been distributed with the program")
 
@@ -1134,8 +1037,7 @@ def cli():
 
     profile_parser = subparsers.add_parser("profile", help="Parse data into the database from one or more sequence files")
     profile_parser.add_argument("-v", "--verbose", help="Prints warnings to the console by default", default=0, action="count")
-    profile_parser.add_argument("-pg", "--postgres-connection", type=str, help="A postgresql connection string, of the format postgres://user:password@host:port/dbname", required=True)
-    profile_parser.add_argument("-p", "--parallel", type=int, default=1, choices=list(range(1, cpu_count()+1)), help="Shred k-mers from reads in parallel")
+    profile_parser.add_argument("-p", "--parallel", type=int, default=1, help="Shred k-mers from reads in parallel")
     profile_parser.add_argument("-pq", "--parallel-fastq", type=int, default=1, help="The number of blocks to read in parallel for reading fastqs")
     profile_parser.add_argument("--batch-size", type=int, default=100000, help="Number of updates to issue per batch to PostgreSQL while counting")
     profile_parser.add_argument("-b", "--fastq-block-size", type=int, default=100000, help="Number of reads to load in memory at once for processing")
@@ -1167,7 +1069,7 @@ def cli():
     view_parser.set_defaults(func=view)
 
 
-    matrix_parser = subparsers.add_parser("matrix", help="Generate a reduced-dimensionality matrix of the n * 4^k (sample x k-mer) data matrix.")
+    matrix_parser = subparsers.add_parser("matrix", help="Generate a reduced-dimensionality matrix of the 4^k * n (k-mers x samples) data matrix.")
     matrix_parser.add_argument("-v", "--verbose", help="Prints warnings to the console by default", default=0, action="count")
     matrix_parser.add_argument("--with-index", default=False, action="store_true", help="Print the row indices as well")
     matrix_parser.add_argument("-d", "--delimiter", default="\t", type=str, help="The choice of delimiter to parse the input .tsv with. DEFAULT: '\t'")
@@ -1180,7 +1082,7 @@ def cli():
 
     matrix_parser.add_argument("--perplexity", default=5, type=int, help="The choice of the perplexity for t-SNE based dimensionality reduction")
     matrix_parser.add_argument("method", choices=["PCA", "tSNE", "Normalized", "Unnormalized"], default=None, help="Choice of distance metric between two profiles")
-    matrix_parser.add_argument("kdb", nargs="+", type=str, metavar="<.kdb>", help="Two or more .kdb files")
+    matrix_parser.add_argument("input", nargs="*", default=[], metavar="<kdbfile1 kdbfile2 ...|input.tsv|STDIN>", help="Two or more .kdb files, or another count matrix in tsv/csv")
     matrix_parser.set_defaults(func=get_matrix)
     
     # rarefy_parser = subparsers.add_parser("rarefy", help="Generate rarefaction information using ecopy.diversity.rarefy for the supplied .kdb files")
@@ -1241,7 +1143,7 @@ def cli():
         "spearman",
         "sqeuclidean",
         "yule"], default="correlation", help="Choice of distance metric between two profiles")
-    dist_parser.add_argument("kdb", nargs="+", type=str, metavar="<kdbfile1 kdbfile2 ...>", help="Two or more .kdb files")
+    dist_parser.add_argument("input", nargs="*", default=[], metavar="<kdbfile1 kdbfile2 ...|input.tsv|STDIN>", help="Two or more .kdb files, or another count matrix in tsv/csv")
     dist_parser.set_defaults(func=distances)
 
 
