@@ -34,21 +34,18 @@ from builtins import open as _open
 
 import jsonschema
 from Bio import SeqIO, bgzf
-import boto3
-sys.path.append('..')
 
-from kmerdb import kmer, database, util, config
+#sys.path.append('..')
+
+from kmerdb import kmer, util, config
 
 # Logging configuration
 import logging
 logger = logging.getLogger(__file__)
 # S3 configuration
-s3 = boto3.resource('s3')
-s3client = boto3.client('s3')
 s3prefix = "s3://"
 
 
-
 def is_all_fasta(filenames):
     """Tests if all the strings in a list are fasta format.
     
@@ -73,6 +70,12 @@ def _s3_file_download(self, seqpath, temporary=True):
     :returns: The location of a downloaded gennomic Fasta file
     :rtype: str
     """
+    import boto3
+    s3 = boto3.resource('s3')
+    s3client = boto3.client('s3')
+
+
+
     if type(seqpath) is not str:
         raise TypeError("kmerdb.fileutil.SeqReader.__s3_file_download expects a str 'seqpath' as its first positional argument")
     elif seqpath[0:5] != s3prefix:
@@ -435,16 +438,22 @@ class KDBWriter(bgzf.BgzfWriter):
         logger.info("Constructing a new .kdb file '{0}'...".format(self._handle.name))
         yaml.add_representer(OrderedDict, util.represent_ordereddict)
 
-        metadata_bytes = bgzf._as_bytes(yaml.dump(self.metadata, sort_keys=False))
-        metadata_plus_delimiter_in_bytes = metadata_bytes + bgzf._as_bytes(config.header_delimiter)
+        metadata_bytes = bytes(yaml.dump(self.metadata, sort_keys=False), 'utf-8')
+        metadata_plus_delimiter_in_bytes = metadata_bytes + bytes(config.header_delimiter, 'utf-8')
         self.metadata["metadata_blocks"] = math.ceil( sys.getsizeof(metadata_plus_delimiter_in_bytes) / ( 2**16 ) ) # First estimate
-        metadata_bytes = bgzf._as_bytes(yaml.dump(self.metadata, sort_keys=False))
-        metadata_bytes = metadata_bytes + bgzf._as_bytes(config.header_delimiter)
+        metadata_bytes = bytes(yaml.dump(self.metadata, sort_keys=False), 'utf-8')
+        metadata_bytes = metadata_bytes + bytes(config.header_delimiter, 'utf-8')
         self.metadata["metadata_blocks"] = math.ceil( sys.getsizeof(metadata_bytes) / ( 2**16 ) ) # Second estimate
+        metadata_bytes = bytes(yaml.dump(self.metadata, sort_keys=False), 'utf-8')
+        metadata_bytes = metadata_bytes + bytes(config.header_delimiter, 'utf-8')
         logger.info("Writing the {0} metadata blocks to the new file".format(self.metadata["metadata_blocks"]))
         logger.debug(self.metadata)
         logger.debug("Header is being written as follows:\n{0}".format(yaml.dump(self.metadata, sort_keys=False)))
 
+        # 01-01-2022 This is still not a completely functional method to write data to bgzf through the Bio.bgzf.BgzfWriter class included in BioPython
+        # I've needed to implement a basic block_writer, maintaining compatibility with the Biopython bgzf submodule.
+        #self.write(bytes(yaml.dump(metadata, sort_keys=False), 'utf-8'))
+        
         for i in range(self.metadata["metadata_blocks"]):
             metadata_slice = metadata_bytes[:65536]
             metadata_bytes = metadata_bytes[65536:]
@@ -454,42 +463,3 @@ class KDBWriter(bgzf.BgzfWriter):
         self._buffer = b""
         self._handle.flush()
         
-    # def write(self, data):
-    #     if not isinstance(data, str):
-    #         raise TypeError("kdb.fileutil.KDBWriter.write() expects a str as its first positional argument")
-    #     else:
-    #         data = data.encode('latin-1')
-
-    #     data_len = len(data)
-    #     if len(self._buffer) + data_len < 65536:
-    #         self._buffer += data
-    #     else:
-    #         self._buffer += data
-    #         while len(self._buffer) >= 65536:
-    #             self._write_block(self._buffer[:65536])
-    #             self._buffer = self._buffer[65536:]
-    #         self.flush()
-
-    # def write_block(self, recs):
-    #     if type(recs) is not list:
-    #         raise TypeError("kdb.fileutil.KDBWriter.write_block() expects a list as its first positional argument")
-
-    #     self._block_size = bgzf._as_bytes(self._buffer).__sizeof__()
-    #     while len(recs):
-        
-    #         while self._block_size < 65530:
-    #             newline = "\t".join(recs.pop()) + "\n"
-
-    #             if self._block_size + bgzf.as_bytes(newline).__sizeof__() < 65530:
-    #                 self._buffer += newline
-    #                 self._block_size += bgzf._as_bytes(newline).__sizeof__()
-    #             else:
-    #                 self._write_block(bgzf._as_bytes(self._buffer))
-    #                 self._buffer = b""
-    #                 self._handle.flush()
-    #                 self._buffer = newline
-    #                 self._block_size = bgzf._as_bytes(newline).__sizeof__()
-    #     self._write_block(bgzf._as_bytes(self._buffer))
-    #     self._buffer = b""
-    #     self._handle.flush()
-        
