@@ -26,19 +26,21 @@ from math import ceil
 from itertools import chain, repeat
 from concurrent.futures import ThreadPoolExecutor
 
-from sqlalchemy.orm import sessionmaker
-from sqlalchemy.orm.attributes import flag_modified
-from sqlalchemy.ext.declarative import declarative_base
-from sqlalchemy import Table, Column, Integer, String, MetaData, ForeignKey, Sequence, JSON, Boolean
+# from sqlalchemy.orm import sessionmaker
+# from sqlalchemy.orm.attributes import flag_modified
+# from sqlalchemy.ext.declarative import declarative_base
+# from sqlalchemy import Table, Column, Integer, String, MetaData, ForeignKey, Sequence, JSON, Boolean
 
-from psycopg2 import sql
+#from psycopg2 import sql
 import tempfile
+import numpy as np
+
 
 #import threading
 from multiprocessing import Pool
 
 
-from kmerdb import seqparser, database, kmer
+from kmerdb import seqparser, kmer
 
 import logging
 logger = logging.getLogger(__file__)
@@ -46,12 +48,8 @@ logger = logging.getLogger(__file__)
 
 
 
-Base = declarative_base()
 
-
-
-
-def parsefile(filepath:str, k:int, connection_string:str, p:int=1, rows_per_batch:int=100000, b:int=50000, n:int=1000, stranded:bool=True, all_metadata:bool=False):
+def parsefile(filepath:str, k:int, p:int=1, rows_per_batch:int=100000, b:int=50000, n:int=1000, stranded:bool=True, all_metadata:bool=False):
     """Parse a single sequence file in blocks/chunks with multiprocessing support
 
     :param filepath: Path to a fasta or fastq file
@@ -64,8 +62,8 @@ def parsefile(filepath:str, k:int, connection_string:str, p:int=1, rows_per_batc
     :type b: int
     :param stranded: Strand specificity argument for k-mer shredding process
     :type stranded: bool
-    :returns: (db, header_dictionary) header_dictionary is the file's metadata for the header block
-    :rtype: (kdb.database.SqliteKdb, dict)
+    :returns: (counts, header_dictionary, nullomers) header_dictionary is the file's metadata for the header block
+    :rtype: (numpy.ndarray, dict, list)
 
     """
     if type(filepath) is not str:
@@ -74,8 +72,6 @@ def parsefile(filepath:str, k:int, connection_string:str, p:int=1, rows_per_batc
         raise OSError("kmerdb.parse.parsefile could not find the file '{0}' on the filesystem".format(filepath))
     elif type(k) is not int:
         raise TypeError("kmerdb.parse.parsefile expects an int as its second positional argument")
-    elif type(connection_string) is not str:
-        raise TypeError("kmerdb.parse.parsefile expects a str as its third positional argument")
     elif type(p) is not int:
         raise TypeError("kmerdb.parse.parsefile expects the keyword argument 'p' to be an int")
     elif type(b) is not int:
@@ -84,13 +80,7 @@ def parsefile(filepath:str, k:int, connection_string:str, p:int=1, rows_per_batc
         raise TypeError("kmerdb.parse.parsefile expects the keyword argument 'n' to be an int")
     elif type(stranded) is not bool:
         raise TypeError("kmerdb.parse.parsefile expects the keyword argument 'stranded' to be a bool")
-    # Create temporary SQLite3 database file for on-disk k-mer counting
-    # temp = tempfile.NamedTemporaryFile(mode="w+", suffix=".sqlite3", delete=False)
-    # logger.debug("Creating temporary database to tally k-mers: '{0}'".format(temp.name))
-    # temp.close()
 
-    logger.debug("Creating empty database table to store the k-mers counts.")
-    db = database.PostgresKdb(k, connection_string, filename=filepath)
     from kmerdb import kmer
 
 
@@ -98,6 +88,7 @@ def parsefile(filepath:str, k:int, connection_string:str, p:int=1, rows_per_batc
     data = {} # This is the dictionary of tuples, keyed on k-mer id, and containing 3-tuples ('kmer_id', 'read/start/reverse')
     keys = set()
     rows = []
+    total_kmers = 4**k
     nullomers = set()
     try:
         # Build fasta/fastq parser object to stream reads into memory
@@ -108,7 +99,8 @@ def parsefile(filepath:str, k:int, connection_string:str, p:int=1, rows_per_batc
         if not fasta: # Cannot process fast files in parallel due to spawning issue.
             pool = Pool(processes=p) # A multiprocessing pool of depth 'p'
 
-
+        # Initialize the kmer array
+        counts = np.zeros(total_kmers, dtype='uint')
         # Instantiate the kmer class
         Kmer = kmer.Kmers(k, strand_specific=stranded, fasta=fasta, all_metadata=all_metadata) # A wrapper class to shred k-mers with
 
@@ -119,11 +111,12 @@ def parsefile(filepath:str, k:int, connection_string:str, p:int=1, rows_per_batc
         else:
             logger.debug("Skipping the block size assertion for fasta files")
         logger.info("Read {0} sequences from the {1} seqparser object".format(len(recs), "fasta" if fasta else "fastq"))
-
+        all_kmer_metadata = list([] for x in range(total_kmers))
         while len(recs): # While the seqprsr continues to produce blocks of reads
             # Run each read through the shred method
             num_recs = len(recs)
-
+            # Initialize an all_kmer_metadata list
+            kmer_metadata = []
             if fasta:
                 list_of_dicts = list(map(Kmer.shred, recs))
             else:
@@ -134,6 +127,7 @@ def parsefile(filepath:str, k:int, connection_string:str, p:int=1, rows_per_batc
             #logger.debug("Everything in list_of_dicts is perfect, everything past here is garbage")
             
             # Flatmap to 'kmers', the dictionary of {'id': read_id, 'kmers': [ ... ]}
+            logger.debug("\n\nAcquiring linked-list of all k-mer ids from {0} sequence records...\n\n".format(num_recs))
             kmer_ids = list(chain.from_iterable(map(lambda x: x['kmers'], list_of_dicts)))
             #logger.debug(kmer_ids)
             logger.debug("Initial k-mers as a dictionary from each of {0} sequences".format(num_recs))
@@ -211,25 +205,9 @@ def parsefile(filepath:str, k:int, connection_string:str, p:int=1, rows_per_batc
             # Assert that all list lengths are equal before adding metadata to k-mers
             if all_metadata is True and len(kmer_ids) == len(reads) and len(reads) == len(starts) and len(starts) == len(reverses):
                 N = len(starts)
-
-
                 
-                data = list(zip(kmer_ids, reads, starts, reverses))
+                kmer_metadata = list(zip(kmer_ids, reads, starts, reverses))
                 # Everything is in the right order
-                # logger.debug("num k-mer ids: {0}".format(len(kmer_ids)))
-                # logger.debug("K-mer id types: {0}".format(type(kmer_ids[0])))
-                # logger.debug("Example: {0}".format(kmer_ids[0]))
-                # logger.debug("Num reads: {0}".format(len(reads)))
-                # logger.debug("reads type: {0}".format(type(reads[0])))
-                # logger.debug("Example: {0}".format(reads[0]))
-                # logger.debug("Num reverses: {0}".format(len(reverses)))
-                # logger.debug("reverse type: {0}".format(type(reverses[0])))
-                # logger.debug("Example: {0}".format(reverses[0]))
-                # raise RuntimeError("Deciding whether to set a dictionary, or a 4x? array")
-                logger.debug("Appended {0} records to rows".format(N))
-
-                rows += data 
-
                 logger.warning("Dumping all metadata into the .kdb file eventually. This could be expensive...")
 
             elif not all_metadata and len(reads) == 0 and len(starts) == 0 and len(reverses) == 0:
@@ -248,184 +226,44 @@ def parsefile(filepath:str, k:int, connection_string:str, p:int=1, rows_per_batc
             # On disk k-mer counting
             # Thank you, this was brilliant
             # https://stackoverflow.com/a/9294062/12855110
+            # 01-01-2022 This has been removed in favor of in-memory counting
             num_kmers = len(kmer_ids)
 
             if num_kmers == 0:
                 raise ValueError("No k-mers to add. Something likely went wrong. Please report to the issue tracker")
             else:
-                # for kmer_id in kmer_ids:
-                #     db.conn.execute("UPDATE {0} SET count = count + 1 WHERE id = $1".format(db._tablename),
-                inserts = list(map(lambda x: (x+1,), kmer_ids))
+                sys.stderr.write("Accumulating all k-mers from this set of records...")
+                for kmer in kmer_ids:
+                    counts[kmer] += 1
+            # all_kmer_metadata
+            if all_metadata:
+                for single_kmer_id, read, start, reverse in kmer_metadata:
+                    all_kmer_metadata[single_kmer_id].append((read, start, reverse))
 
-                batches = ceil(num_kmers/rows_per_batch)
-                t0 = datetime.now()
-                for i in range(batches):
 
-                    try:
-                        with db.conn.begin():
-
-                            db.conn.execute("UPDATE {0} SET count = count + 1 WHERE id IN (%s)".format(db._tablename), inserts[i*rows_per_batch:(i*rows_per_batch+rows_per_batch)])
-
-                    except Exception as e:
-                        logger.error(db.conn)
-                        raise e
-
-                    t1 = datetime.now()
-                    d = (t1 - t0).seconds
-                    logger.info("\n\nLoaded {0} k-mers in batch {1} transaction ({2} rows/second)".format(rows_per_batch, i, rows_per_batch/(d+0.1)))
-                    t0 = t1
-                    t1 = None
-                    
-            
             recs = [r for r in seqprsr] # The next block of exactly 'b' reads
             # This will be logged redundantly with the sys.stderr.write method calls at line 141 and 166 of seqparser.py (in the _next_fasta() and _next_fastq() methods)
             #sys.stderr("\n")
             logger.info("Read {0} more records from the {1} seqparser object".format(len(recs), "fasta" if fasta else "fastq"))
-        if all_metadata:
-            sys.stderr.write("Writing all metadata keys for each k-mer's relationships to reads into the SQLite3 database...\n")
+        # Get nullomers
+        # only nullomer counts
+        unique_kmers = int(np.count_nonzero(counts))
+        num_nullomers = total_kmers - unique_kmers
+        is_nullomer = np.where(counts == 0)
+        nullomers = []
+        for i, j in enumerate(is_nullomer):
+            if j is True:
+                nullomers.append(i)
+        seqprsr.total_kmers = int(np.sum(counts))
+        seqprsr.unique_kmers = unique_kmers
+        seqprsr.nullomers = num_nullomers
 
-            unique_kmer_ids = list(set(list(map(lambda x: x[0], rows))))
-
-
-
-            # If we round up to the nearest page, we should have 'pages' number of pages
-            # and we'd read n on each page.
-            kid = 0
-            Session = sessionmaker(bind=db._engine)
-            session = Session()
-            class Kmer(Base):
-                __tablename__ = db._tablename
-
-                id = Column(Integer, primary_key=True)
-                count = Column(Integer)
-                starts = Column(JSON)
-                reverses = Column(JSON)
-                seqids = Column(JSON)
-
-            for i in range(len(unique_kmer_ids)):
-                kid = kmer_ids[i] # FOr SQLlite 1-based indexing
-                logger.debug("Beginning to commit {0} to the database".format(kid))
-                sys.stderr.write("\n")
-                kmers = [x for x in rows if x[0] == kid]
-                logger.debug("Located {0} relationships involving k-mer {1}".format(len(kmers), kid))
-                logger.debug("        === M E S S A G E ===")
-                logger.debug("=====================================")
-                logger.debug("beginning to process all records of the {0} k-mer".format(kid))
-
-
-                row = session.query(Kmer).filter_by(id=kid).first()
-                if row is None:
-                    logger.error(kid)
-                    raise RuntimeError("Could not locate k-mer with id {0} in the Postgres table".format(kid))
-
-
-                logger.debug("Before: {0}".format(len(row.reverses)))
-                row.seqids = list(map(lambda y: y[1], kmers))
-                row.starts = list(map(lambda y: y[2], kmers))
-                row.reverses = list(map(lambda y: y[3], kmers))
-                #flag_modified(Kmer, 'seqids')
-                #flag_modified(Kmer, 'starts')
-                #flag_modified(Kmer, 'reverses')
-
-
-                logger.debug("After: {0}".format(len(row.reverses)))
-                session.add(row)
-
-                if i % rows_per_batch == 0:
-                    session.commit()
-                    logger.info("Transaction completed for the {0} kmer.".format(kid+1))
-            session.commit()
-            logger.debug("Sleeping...")
-            time.sleep(200)
-            session.commit()
-
-            logger.debug("===================================")
-            logger.debug("Example record with metadata:")
-            result = session.query(Kmer).filter_by(id=kid+1).first()
-            logger.debug(result.__dict__)
-            logger.debug("===================================")
-            session.close()
-        seqprsr.nullomers = db._get_nullomers() # Calculate nullomers at the end
-        seqprsr.total_kmers = db._get_sum_counts() # The total number of k-mers processed
-        # Get nullomer ids
-        res = list(db.conn.execute("SELECT id FROM {0} WHERE count = 0".format(db._tablename)).fetchall())
-        if type(res) is not list or not all(type(x[0]) is int for x in res):
-            logger.error("{0}\nSELECT id FROM kmers WHERE count = 0".format(res))
-            logger.error("Type of res: {0}".format(type(res)))
-            logger.error("Types of values: {0}".format([type(x[0]) for x in res]))
-            logger.error("The result of the query was not a list of singleton tuples of ints")
-            raise ValueError("PostgreSQL database query returned unexpected data types")
-        nullomers = list(map(lambda x: x[0], res))
-
-    finally:
-        sys.stderr.write("\n\n\nFinished counting k-mers{0} from '{1}' into the database...\n\n\n".format(' and metadata' if all_metadata else '', filepath))
-        if db.conn is not None:
-            db.conn.close()
-
-    return db._tablename, seqprsr.header_dict(), nullomers
-
-
-# class MetadataAppender:
-#     def __init__(self, ids:list, reads:list, starts:list, reverses:list, data:dict={}):
-#         if type(ids) is not list or not all(type(i) is int for i in ids):
-#             raise TypeError("kmerdb.parse.MetadataAppender.__init__() expects a list of ints as its first positional argument")
-#         elif type(reads) is not list or not all(type(r) is str for r in reads):
-#             raise TypeError("kmerdb.parse.MetadataAppender.__init__() expects a list of strings as its second positional argument")
-#         elif type(starts) is not list or not all(type(s) is int for s in starts):
-#             raise TypeError("kmerdb.parse.MetadataAppender.__init__() expects a list of ints as its third positional argument")
-#         elif type(reverses) is not list or not all(type(r) is bool for r in reverses):
-#             raise TypeError("kmerdb.parse.MetadataAppender.__init__() expects a list of bools as its fourth positional argument")
-#         elif type(data) is not dict:
-#             raise TypeError("kmerdb.parse.MetadataAppender.__init__{} expects a data dictionary as its fifth positional argument")
-
-#         self.data = data
-#         self.ids = ids
-#         self.reads = reads
-#         self.starts = starts
-#         self.reverses = reverses
-
-#     def appendRow(self, i):
-#         """
-#         :param i: 
-#         :type i: int
-#         :returns: 
-#         :rtype: tuple
-
-#         """
-#         if type(i) is not int:
-#             raise TypeError("kmerdb.MetadataAppender.appendRow expects an int as its first positional argument")
-#         logger.info("Processing the {0} k-mer by appending read relations, start offsets, and reverse 3-tuples to the metadata column".format(i))
-#         row = (self.ids[i], self.reads[i], self.starts[i], self.reverses[i])
-#         def helper(row, data, j):
-#             data = yaml.safe_load(data)
-#             if type(data) is str:
-#                 logger.error(data)
-#                 raise ValueError("data: '{0}' with j={1} parsed as a string".format(data, j))
-#             elif type(data) is list:
-#                 data.append(row[j])
-#             else:
-#                 logger.debug(data)
-#                 raise RuntimeError("data cannot parse as something other than list...")
-#             return data
         
-#         # with self.conn.begin():
-#         #     seqids_data = self.conn.execute("SELECT seqids FROM kmers WHERE id = ?", self.ids[i])
-#         #     seqids = helper(row, seqids_data, 1)
-#         #     self.conn.execute("UPDATE kmers SET seqids = ? WHERE id = ?", json.dumps(seqids), self.ids[i])
+    finally:
+        sys.stderr.write("\n\n\nFinished counting k-mers{0} from '{1}'...\n\n\n".format(' and metadata' if all_metadata else '', filepath))
 
+    return counts, seqprsr.header_dict(), nullomers, all_kmer_metadata
 
-#         # with self.conn.begin():
-#         #     starts_data = self.conn.execute("SELECT starts FROM kmers WHERE id = ?", self.ids[i])
-#         #     starts = helper(row, starts_data, 2)
-#         #     self.conn.execute("UPDATE kmers SET starts = ? WHERE id = ?", json.dumps(starts), self.ids[i])
-
-
-#         # with self.conn.begin():
-#         #     reverses_data = self.conn.execute("SELECT reverses FROM kmers WHERE id = ?", self.ids[i])
-#         #     reverses = helper(row, reverses_data, 3)
-#         #     self.conn.execute("UPDATE kmers SET reverses = ? WHERE id = ?", json.dumps(reverses), self.ids[i])
-
-#         return row
 
 
 
@@ -441,6 +279,6 @@ class Parseable:
         :type data: dict
         :returns: (db, m, n)
         """
-        return parsefile(filename, self.arguments.k, self.arguments.postgres_connection, p=self.arguments.parallel_fastq, rows_per_batch=self.arguments.batch_size, b=self.arguments.fastq_block_size, n=self.arguments.n, stranded=self.arguments.strand_specific, all_metadata=self.arguments.all_metadata)
+        return parsefile(filename, self.arguments.k, p=self.arguments.parallel_fastq, rows_per_batch=self.arguments.batch_size, b=self.arguments.fastq_block_size, n=self.arguments.n, stranded=self.arguments.strand_specific, all_metadata=self.arguments.all_metadata)
 
 
